#!/bin/bash
# Copyright (C) 2009-2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

# This file is part of collect-agent

# collect-agent is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

# ------------------------------------------------------------------------------
# Function: check_traffic_date
#           Checks if the date written in TRAFFIC file is today or creates a
#           new TRAFFIC if its not.
function check_traffic_date()
{
    # If there is no TRAFFIC (probably its the first run), then just create a
    # new one.
    if ! test -f "$TRAFFIC"; then
    	echo $(date +%F) > $TRAFFIC
        return
    fi

    local CURRENT_DATE=$(date -d"$(date +%F)" +%s)
    local TRAFFIC_DATE=$(date -d"$(head -n1 $TRAFFIC)" +%s)

    # If it midnight (INTERVALID = 287), then consider current date as
    # yesterday (subtract one day from current date). Thats because the
    # data collect belongs to an interval beginning yesterday 23:55 and
    # only ending today, at midnight.
    if test $INTERVALID -eq 287; then
        CURRENT_DATE=$(($CURRENT_DATE - 86400))
    fi

    # Finally, create a new TRAFFIC (keeping the previous one as
    # PREVIOUS_TRAFFIC) if the dates doesnt match.
    if test $CURRENT_DATE -ne $TRAFFIC_DATE; then
    	mkdir -p "./previous"
        cp $TRAFFIC $PREVIOUS_TRAFFIC
        echo $(date +%F) > $TRAFFIC
    fi
}

#------------------------------------------------------------------------------
# Function: get_interval_id
#           Returns a number identifying the current five minutes interval
function get_interval_id()
{
    INTERVALID=$(date +%H:%M | awk -F':' '{print $1 * 12 + \
                                           int($2 / 5 + 0.5) - 1}')

    # INTERVALID equal to -1 means midnight 00:00, but midnight needs
    # to be the last id possible, so force it to 287
    if test $INTERVALID -eq -1; then
        INTERVALID=287
    fi
}

# ------------------------------------------------------------------------------
# Initialization

# set PREFIX
export PREFIX="$(readlink -f "$(dirname $0)/../")"

# run datasid-common.sh
source "$(readlink -f "$(dirname $0)")/datasid-common.sh" || exit 1

if test -z "${PREFIX}"; then
    date +"%F %T - ERROR: Prefix not set."
    exit 2
fi

cd ${DATADIR}

# Collected data from the current day
TRAFFIC="./traffic.txt"

# Collected data on the last day before the machine is disconnected
PREVIOUS_TRAFFIC="./previous/traffic-$(date +%F).txt"

# Stored interfaces from the last run
INTERFACES="./interfaces.txt"

# Find binaries
IP=$(which ip)
test $? -ne 0 && IP="/sbin/ip"

IPTABLES=$(which iptables)
test $? -ne 0 && IPTABLES="/sbin/iptables"

test -x $IP || exit 1
test -x $IPTABLES || (IPTABLES="/usr/sbin/iptables" && test -x $IPTABLES) || exit 1


#------------------------------------------------------------------------------
# Calculate bytes and packets counts and print to TRAFFIC

get_interval_id
if test $? -ne 0; then exit 2; fi

# Check if TRAFFIC is fresh
check_traffic_date
if test $? -ne 0; then exit 3; fi

INPUT_COUNT=$(sudo $IPTABLES -nvxL INPUT | grep INPUT_COUNTS | \
              awk 'BEGIN{pkts=0; bytes=0} \
                   {pkts+=$1; bytes+=$2} \
                   END{print pkts,bytes}')

OUTPUT_COUNT=$(sudo $IPTABLES -nvxL OUTPUT | grep OUTPUT_COUNTS | \
               awk 'BEGIN{pkts=0; bytes=0} \
                    {pkts+=$1; bytes+=$2} \
                    END{print pkts,bytes}')

echo "$INTERVALID $INPUT_COUNT $OUTPUT_COUNT" >> $TRAFFIC


#------------------------------------------------------------------------------
# Create or update iptables rules (used for counting) based on the interfaces

# Remove all current rules
if test -f "$INTERFACES"; then
    while IFS=$' ' read -a net; do
        sudo $IPTABLES -D INPUT -i ${net[0]} ! -s ${net[1]} -m comment \
             --comment INPUT_COUNTS 2>/dev/null
        sudo $IPTABLES -D OUTPUT -o ${net[0]} ! -d ${net[1]} -m comment \
             --comment OUTPUT_COUNTS 2>/dev/null
    done < $INTERFACES
fi

# Truncate interfaces file
echo -n > $INTERFACES

# Grab all the interfaces in wich DEFAULT traffic gets routed
TMP=$($IP route list | sed -n 's/^default.*dev \([a-zA-Z0-9]*\).*$/\1/p')

# Find IP address and subnet mask for each interface found
for IF in $TMP; do
    addr_mask=$($IP addr show $IF | sed -n 's/^.*inet \([0-9.]*\/[0-9]*\).*$/\1/p')
    echo "$IF $addr_mask" >> $INTERFACES
done

# Insert new rules
while IFS=$' ' read -a net; do
    sudo $IPTABLES -I INPUT -i ${net[0]} ! -s ${net[1]} -m comment \
         --comment INPUT_COUNTS
    sudo $IPTABLES -I OUTPUT -o ${net[0]} ! -d ${net[1]} -m comment \
         --comment OUTPUT_COUNTS
done < $INTERFACES
