/**
 * The functions on this file return the number of computers of a given
 * 'project' that contacted the server in the last 10 days (green), 10
 * to 30 days (yellow), or more than 30 days (red). You can also provide
 * 'base_date' to refer to the situation in a particular date.
 */


/**
 * Returns the total amount of computers of the given 'project' and their
 * last contact date status.
 *
 * @param in project_enum proj The type of computer/machine
 * @return TABLE(bigint, bigint, bigint)
 */
CREATE OR REPLACE FUNCTION availability_brazil(machtype machine_type_enum) RETURNS
	TABLE ("Verde" bigint, "Amarelo" bigint, "Vermelho" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		SUM(is_green),
		SUM(is_yellow),
		SUM(is_red)
	FROM
		aggr_availability
	WHERE
		machine_type = machtype AND base_date = (SELECT max(base_date) FROM aggr_availability);
END;
$$ language plpgsql;


/**
 * Returns the total amount of computers of the given 'project' in the
 * current date and the last six months.
 *
 * @param in project_enum proj The type of computer/machine
 * @return TABLE(date, bigint, bigint, bigint)
 */
CREATE OR REPLACE FUNCTION availability_brazil_monthly_history(machtype machine_type_enum) RETURNS
	TABLE ("Data" text, "Verde" bigint, "Amarelo" bigint, "Vermelho" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		get_month_name(extract(month from base_date)::integer)::text,
		SUM(is_green),
		SUM(is_yellow),
		SUM(is_red)
	FROM
		aggr_availability
	WHERE
		machine_type = machtype
	GROUP BY
		base_date
	ORDER BY
		base_date;
END;
$$ language plpgsql;


/**
 * Returns the total amount of computers of the given 'project' and their
 * last contact date status grouped by region.
 *
 * @param in project_enum proj The type of computer/machine
 * @return TABLE(varchar, bigint, bigint, bigint)
 */
CREATE OR REPLACE FUNCTION availability_region(machtype machine_type_enum) RETURNS
	TABLE ("Região" varchar, "Verde" bigint, "Amarelo" bigint, "Vermelho" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		region,
		SUM(is_green),
		SUM(is_yellow),
		SUM(is_red)
	FROM
		aggr_availability
	WHERE
		machine_type = machtype AND base_date = (SELECT max(base_date) FROM aggr_availability)
	GROUP BY
		region;
END;
$$ language plpgsql;


/**
 * Returns the total amount of computers of the given 'project' in the
 * current date and the last six months of a given region.
 *
 * @param in project_enum proj The type of computer/machine
 * @param in region_in text The region
 * @return TABLE(date, bigint, bigint, bigint)
 */
CREATE OR REPLACE FUNCTION availability_region_monthly_history(machtype machine_type_enum, region_in text) RETURNS
	TABLE ("Data" text, "Verde" bigint, "Amarelo" bigint, "Vermelho" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		get_month_name(extract(month from base_date)::integer)::text,
		SUM(is_green),
		SUM(is_yellow),
		SUM(is_red)
	FROM
		aggr_availability
	WHERE
		machine_type = machtype AND region = region_in
	GROUP BY
		base_date
	ORDER BY
		base_date;
END;
$$ language plpgsql;


/**
 * Returns the total amount of computers of the given 'project' and their
 * last contact date status grouped by state in a given 'region_in'
 *
 * @param in project_enum proj The type of computer/machine
 * @param in region_in text The region
 * @return TABLE(varchar, bigint, bigint, bigint)
 */
CREATE OR REPLACE FUNCTION availability_state(machtype machine_type_enum, region_in text) RETURNS
	TABLE ("Estado" varchar, "Verde" bigint, "Amarelo" bigint, "Vermelho" bigint) AS $$
BEGIN

	RETURN QUERY SELECT
		state,
		SUM(is_green),
		SUM(is_yellow),
		SUM(is_red)
	FROM
		aggr_availability
	WHERE
		machine_type = machtype AND region = region_in AND base_date = (SELECT max(base_date) FROM aggr_availability)
	GROUP BY
		state;
END;
$$ language plpgsql;


/**
 * Returns the total amount of computers of the given 'project' in the
 * current date and the last six months in the given 'states_in'.
 *
 * @param in project_enum proj The type of computer/machine
 * @param in state_in text The state
 * @return TABLE(date, bigint, bigint, bigint)
 */
CREATE OR REPLACE FUNCTION availability_state_monthly_history(machtype machine_type_enum, state_in text) RETURNS
	TABLE ("Data" text, "Verde" bigint, "Amarelo" bigint, "Vermelho" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		get_month_name(extract(month from base_date)::integer)::text,
		SUM(is_green),
		SUM(is_yellow),
		SUM(is_red)
	FROM
		aggr_availability
	WHERE
		machine_type = machtype AND state = state_in
	GROUP BY
		base_date
	ORDER BY
		base_date;
END;
$$ language plpgsql;


/**
 * Returns the total amount of computers of the given 'project' and their
 * last contact date status grouped by city in a given 'state_in'
 *
 * @param in project_enum proj The type of computer/machine
 * @param in state_in text The state
 * @return TABLE(varchar, bigint, bigint, bigint)
 */
CREATE OR REPLACE FUNCTION availability_city(machtype machine_type_enum, state_in text) RETURNS
	TABLE ("Cidade" varchar, "Verde" bigint, "Amarelo" bigint, "Vermelho" bigint) AS $$
BEGIN

	RETURN QUERY SELECT
		city,
		SUM(is_green),
		SUM(is_yellow),
		SUM(is_red)
	FROM
		aggr_availability
	WHERE
		machine_type = machtype AND state = state_in AND base_date = (SELECT max(base_date) FROM aggr_availability)
	GROUP BY
		city
	ORDER BY
		city;
END;
$$ language plpgsql;


/**
 * Returns the total amount of computers of the given 'project'
 * region, state and city grouped by school. 
 *
 * @param in project_enum proj The type of computer/machine
 * @param in region text The region
 * @param in state text The state
 * @param in city text The city
 * @return TABLE(TEXT, macaddr, TEXT, TEXT, TEXT, DATE, INT, BIGINT,
 *  TEXT, BOOLEAN, BOOLEAN, BOOLEAN)
 */
CREATE OR REPLACE FUNCTION availability_report(machtype machine_type_enum, region TEXT, state TEXT, city TEXT) RETURNS TABLE
("load_date" TEXT, "machine" macaddr, "region" TEXT, "state" TEXT, "city" TEXT, "last_contact" DATE,
"days_last_contact" INT, "month_contacts" BIGINT, "telecenter" TEXT, "green" BOOLEAN, "yellow" BOOLEAN, "red" BOOLEAN) AS $$
	SELECT
		to_char((SELECT max(base_date) FROM aggr_availability), 'DD/MM/YYYY'),
		c.macaddr AS macad,
         	region,
		state,
		cy.name,
		c.last_contact,
		days_last_contact,
		(SELECT
			COUNT(*)
		FROM
			fact_contact AS fc
		WHERE
			fc.macaddr = c.macaddr AND
			fc.tl_id = c.tl_id AND
			EXTRACT(MONTH FROM fc.dat_id) = EXTRACT(MONTH FROM CURRENT_DATE) AND
			EXTRACT(YEAR FROM fc.dat_id) = EXTRACT(YEAR FROM CURRENT_DATE)) AS num_contacts_in_month,
		t.name,
		days_last_contact <= 10 AS g,
		days_last_contact > 10 AND days_last_contact <= 30 AS y,
		days_last_contact > 30 AS r
	FROM
		(SELECT
			MAX(dat_id) AS last_contact,
			tl_id,
			cit_id,
			macaddr,
			machine_type,
			CURRENT_DATE - MAX(dat_id) AS days_last_contact
		FROM
			fact_contact
		GROUP BY
			tl_id,
			macaddr,
			cit_id,
			machine_type) c
	INNER JOIN
		dim_city cy 
	ON
		c.cit_id = cy.id
	INNER JOIN
		dim_telecenter t
	ON
		c.tl_id = t.id
	WHERE
		machine_type = $1 AND region = $2 AND state = $3 AND cy.name = $4
	ORDER BY
		c.tl_id;
$$ LANGUAGE SQL;
